@@ -121,6 +121,10 @@ kvm_proclist(kd, what, arg, p, bp, maxcnt)
 
 	kp = &kinfo_proc;
 	kp->ki_structsize = sizeof(kinfo_proc);
+	/*
+	 * Loop on the processes. this is completely broken because we need to be
+	 * able to loop on the threads and merge the ones that are the same process some how.
+	 */
 	for (; cnt < maxcnt && p != NULL; p = LIST_NEXT(&proc, p_list)) {
 		memset(kp, 0, sizeof *kp);
 		if (KREAD(kd, (u_long)p, &proc)) {
@@ -141,9 +145,14 @@ kvm_proclist(kd, what, arg, p, bp, maxcnt)
 			kp->ki_svuid = ucred.cr_svuid;
 			kp->ki_rgid = ucred.cr_rgid;
 			kp->ki_svgid = ucred.cr_svgid;
-			kp->ki_ngroups = ucred.cr_ngroups;
-			bcopy(ucred.cr_groups, kp->ki_groups,
-			    NGROUPS * sizeof(gid_t));
+			kp->ki_cr_flags = ucred.cr_flags;
+			if (ucred.cr_ngroups > KI_NGROUPS) {
+				kp->ki_ngroups = KI_NGROUPS;
+				kp->ki_cr_flags |= KI_CRF_GRP_OVERFLOW;
+			}
+				kp->ki_ngroups = ucred.cr_ngroups;
+			kvm_read(kd, (u_long)ucred.cr_groups, kp->ki_groups,
+			    kp->ki_ngroups * sizeof(gid_t));
 			kp->ki_uid = ucred.cr_uid;
 			if (ucred.cr_prison != NULL) {
 				if (KREAD(kd, (u_long)ucred.cr_prison, &pr)) {
@@ -403,15 +412,13 @@ nopgrp:
 			kp->ki_pri.pri_native = mtd.td_base_pri;
 			kp->ki_lastcpu = mtd.td_lastcpu;
 			kp->ki_wchan = mtd.td_wchan;
+			if (mtd.td_name[0] != 0)
+				strlcpy(kp->ki_ocomm, mtd.td_name, MAXCOMLEN);
 			kp->ki_oncpu = mtd.td_oncpu;
-
-			if (!(proc.p_flag & P_SA)) {
-				kp->ki_pctcpu = 0;
-				kp->ki_rqindex = 0;
-			} else {
-				kp->ki_tdflags = -1;
-				/* All the rest are 0 for now */
-			}
+			if (mtd.td_name[0] != '\0')
+				strlcpy(kp->ki_ocomm, mtd.td_name, sizeof(kp->ki_ocomm));
+			kp->ki_pctcpu = 0;
+			kp->ki_rqindex = 0;
 		} else {
 			kp->ki_stat = SZOMB;
 		}
